﻿// Copyright © Microsoft Corporation. 
// This source is subject to the Microsoft Source License for Silverlight Controls (March 2008 Release).
// Please see http://go.microsoft.com/fwlink/?LinkID=111693 for details.
// All other rights reserved. 

using System;
using System.Windows; 
using System.Windows.Input; 

#if !WPFIMPLEMENTATION 

#if WPF
namespace WPF 
#else
namespace System.Windows.Controls.Test
#endif 
{ 
    /// <summary>
    /// Provides helpers for accessing protected methods of ListBoxItem 
    /// </summary>
    class HelperListBoxItem : ListBoxItem
    { 
        private MouseEventArgs MouseEventArgs { get; set; }
        private MouseButtonEventArgs MouseButtonEventArgs { get; set; }
        public HelperListBoxItem() 
        { 
#if WPF
            Type typeWin32MouseDevice = typeof(MouseDevice).Assembly.GetType("System.Windows.Input.Win32MouseDevice"); 
            MouseDevice mouseDevice = (MouseDevice)Activator.CreateInstance(typeWin32MouseDevice, System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance, null, new object[] { InputManager.Current }, null);
            MouseEventArgs = new MouseEventArgs(mouseDevice, 0);
            MouseButtonEventArgs = new MouseButtonEventArgs(mouseDevice, 0, MouseButton.Left); 
            System.Reflection.FieldInfo fieldInfo = typeof(InputManager).GetField("InputReportEvent", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Static);
            MouseButtonEventArgs.RoutedEvent = (RoutedEvent)fieldInfo.GetValue(null);
#else 
            MouseEventArgs = new MouseEventArgs(); 
            MouseButtonEventArgs = new MouseButtonEventArgs();
#endif 
        }
        public void CallOnGotFocus()
        { 
            OnGotFocus(new RoutedEventArgs());
        }
        public void CallOnLostFocus() 
        { 
            OnLostFocus(new RoutedEventArgs());
        } 
        public void CallOnMouseEnter()
        {
            OnMouseEnter(MouseEventArgs); 
        }
        public void CallOnMouseLeave()
        { 
            OnMouseLeave(MouseEventArgs); 
        }
        public void CallOnMouseLeftButtonDown() 
        {
            OnMouseLeftButtonDown(MouseButtonEventArgs);
        } 
    }
}
 
#endif 
