﻿// Copyright © Microsoft Corporation. 
// This source is subject to the Microsoft Source License for Silverlight Controls (March 2008 Release).
// Please see http://go.microsoft.com/fwlink/?LinkID=111693 for details.
// All other rights reserved. 

using System.Collections.Generic;
using System.Collections.ObjectModel; 
using System.Windows; 
using System.Windows.Controls;
using System.Windows.Shapes; 
using Microsoft.VisualStudio.TestTools.UnitTesting;
#if !WPF
using Microsoft.Silverlight.Testing; 
#endif

#if WPF 
namespace WPF 
#else
namespace System.Windows.Controls.Test 
#endif
{
    /// <summary> 
    /// Regression tests for System.Windows.Controls.ItemsControl
    /// </summary>
    [TestClass] 
    public sealed partial class ItemsControlTest : SilverlightTest 
    {
        [TestMethod] 
        public void ItemCollectionContents()
        {
            ItemsControl itemsControl = new ItemsControl(); 
            itemsControl.Items.Add(new object());
        }
 
        [TestMethod] 
        public void ItemsItemsSourceConsistency()
        { 
            List<object> list = new List<object>();
            list.Add(new object());
            ItemsControl itemsControl = new ItemsControl(); 
            itemsControl.ItemsSource = list;
            Assert.AreEqual(1, itemsControl.Items.Count);
        } 
 
        [TestMethod]
        public void PrepareContainerForItemOverrideCalled() 
        {
            ItemsControlDerived itemsControlDerived = new ItemsControlDerived();
            ObservableCollection<object> items = new ObservableCollection<object>(); 
            itemsControlDerived.ItemsSource = items;

            // This method gets the ItemsControl.*Override overrides called by putting the ListBox in a Panel 
            StackPanel stackPanel = new StackPanel(); 
            stackPanel.Children.Add(itemsControlDerived);
            stackPanel.Measure(new Size { Width = 100, Height = 100 }); 

            items.Add(new Ellipse());
            items.Add(new Rectangle()); 

            Assert.AreEqual(2, itemsControlDerived.CallCount);
        } 
 
        [TestMethod]
        public void AssignArrayToItemsSource() 
        {
            ItemsControl itemsControl = new ItemsControl();
            itemsControl.ItemsSource = "hello there".Split(); 
        }

        [TestMethod] 
        public void AssignListToItemsSource() 
        {
            ItemsControl itemsControl = new ItemsControl(); 
            List<object> list = new List<object>();
            itemsControl.ItemsSource = list;
        } 

        [TestMethod]
        public void CallItemsRemoveAt() 
        { 
            ItemsControl itemsControl = new ItemsControl();
            itemsControl.Items.Add(new Rectangle()); 
            itemsControl.Items.RemoveAt(0);
        }
 
        [TestMethod]
#if !WPF
        [Ignore, Bug("TODO: Remove when Jolt bug 14342 is resolved (ClearContainerForItemOverride's item parameter can be null)")] 
#endif 
        public void ClearContainerForItemOverrideGetsNonNullItem()
        { 
            ItemsControlDerived itemsControlDerived = new ItemsControlDerived();
            ObservableCollection<object> items = new ObservableCollection<object>();
            itemsControlDerived.ItemsSource = items; 

            // This method gets the ItemsControl.*Override overrides called by putting the ListBox in a Panel
            StackPanel stackPanel = new StackPanel(); 
            stackPanel.Children.Add(itemsControlDerived); 
            stackPanel.Measure(new Size { Width = 100, Height = 100 });
 
            items.Add(new Ellipse());
            items.Clear();
        } 

        private class ItemsControlDerived : ItemsControl
        { 
            public int CallCount { get; set; } 
            protected override bool IsItemItsOwnContainerOverride(object item)
            { 
                return item is Ellipse;
            }
            protected override void PrepareContainerForItemOverride(System.Windows.DependencyObject element, object item) 
            {
                base.PrepareContainerForItemOverride(element, item);
                CallCount++; 
            } 
            protected override void ClearContainerForItemOverride(DependencyObject element, object item)
            { 
                Assert.IsNotNull(item);
                base.ClearContainerForItemOverride(element, item);
            } 
        }
    }
} 
