﻿// Copyright © Microsoft Corporation. 
// This source is subject to the Microsoft Source License for Silverlight Controls (March 2008 Release).
// Please see http://go.microsoft.com/fwlink/?LinkID=111693 for details.
// All other rights reserved. 

using System;
using System.Windows; 
using System.Windows.Controls; 
using System.Windows.Input;
using Microsoft.VisualStudio.TestTools.UnitTesting; 

#if !WPF
using Microsoft.Silverlight.Testing; 
#endif

#if WPF && !WPFIMPLEMENTATION 
namespace WPF 
#else
namespace System.Windows.Controls.Test 
#endif
{
    /// <summary> 
    /// Unit tests for System.Windows.Controls.ListBoxItem
    /// </summary>
    [TestClass] 
    public sealed partial class ListBoxItemTest : SilverlightControlTest 
    {
        [TestMethod] 
        public void Constructor()
        {
            ListBoxItem listBoxItem = new ListBoxItem(); 
            Assert.IsNotNull(listBoxItem);
        }
 
        [TestMethod] 
        public void InitialValues()
        { 
            ListBoxItem listBoxItem = new ListBoxItem();
            Assert.IsFalse(listBoxItem.IsSelected);
        } 

        [TestMethod]
        public void BasicIsSelected() 
        { 
            ListBoxItem listBoxItem = new ListBoxItem();
            listBoxItem.IsSelected = true; 
            Assert.IsTrue(listBoxItem.IsSelected);
            listBoxItem.IsSelected = false;
            Assert.IsFalse(listBoxItem.IsSelected); 
        }

        [TestMethod] 
        public void ApplyTemplate() 
        {
            ListBoxItem listBoxItem = new ListBoxItem(); 
            ApplyTemplateAndParent(listBoxItem);
        }
 
        [TestMethod]
        [Asynchronous]
        public void IsSelectedWithTemplate() 
        { 
            ListBoxItem listBoxItem = new ListBoxItem();
            FrameworkElement frameworkElement = ApplyTemplateAndParent(listBoxItem); 
            CreateAsyncTest(frameworkElement, delegate
            {
                listBoxItem.IsSelected = true; 
                Assert.IsTrue(listBoxItem.IsSelected);
                listBoxItem.IsSelected = false;
                Assert.IsFalse(listBoxItem.IsSelected); 
            }); 
        }
 
#if !WPFIMPLEMENTATION

        [TestMethod] 
        public void GotAndLostFocus()
        {
            HelperListBoxItem helperListBoxItem = new HelperListBoxItem(); 
            ApplyTemplateAndParent(helperListBoxItem); 
            helperListBoxItem.CallOnGotFocus();
            helperListBoxItem.CallOnLostFocus(); 
        }

        [TestMethod] 
        [Asynchronous]
        public void MouseEnterDownAndLeave()
        { 
            HelperListBoxItem helperListBoxItem = new HelperListBoxItem(); 
            FrameworkElement frameworkElement = ApplyTemplateAndParent(helperListBoxItem);
            CreateAsyncTest(frameworkElement, delegate 
            {
                helperListBoxItem.CallOnMouseEnter();
                helperListBoxItem.CallOnMouseLeftButtonDown(); 
                helperListBoxItem.CallOnMouseLeave();
            });
        } 
 
        [TestMethod]
        [Asynchronous] 
        public void TransitionAllStates()
        {
            HelperListBoxItem helperListBoxItem = new HelperListBoxItem(); 
            FrameworkElement frameworkElement = ApplyTemplateAndParent(helperListBoxItem);
            CreateAsyncTest(frameworkElement, delegate
            { 
                helperListBoxItem.CallOnMouseEnter(); 
                helperListBoxItem.CallOnMouseLeave();
                helperListBoxItem.IsSelected = true; 
                helperListBoxItem.CallOnMouseEnter();
                helperListBoxItem.CallOnMouseLeave();
                HelperListBox helperListBox = helperListBoxItem.ParentListBox as HelperListBox; 
                Assert.IsNotNull(helperListBox);
                helperListBox.CallOnGotFocus();
                helperListBoxItem.CallOnMouseEnter(); 
                helperListBoxItem.CallOnMouseLeave(); 
            });
        } 

#endif
 
        private static FrameworkElement ApplyTemplateAndParent(ListBoxItem listBoxItem)
        {
            string ListBoxItemTemplate = 
                "<ControlTemplate " + 
#if WPF
                    "xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"" + 
#else
                    "xmlns=\"http://schemas.microsoft.com/client/2007\"" +
#endif 
                    " xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\">" +
                    "<StackPanel x:Name=\"RootElement\">" +
                        "<StackPanel.Resources>" + 
                            "<Storyboard x:Key=\"Normal State\"/>" + 
                            "<Storyboard x:Key=\"Unfocused Selected State\"/>" +
                            "<Storyboard x:Key=\"Normal Selected State\"/>" + 
                            "<Storyboard x:Key=\"MouseOver State\"/>" +
                            "<Storyboard x:Key=\"MouseOver Unfocused Selected State\"/>" +
                            "<Storyboard x:Key=\"MouseOver Selected State\"/>" + 
                        "</StackPanel.Resources>" +
                        "<Rectangle x:Name=\"FocusVisualElement\"/>" +
                    "</StackPanel>" + 
                "</ControlTemplate>"; 

            ControlTemplate template = LoadXamlFromString<ControlTemplate>(ListBoxItemTemplate); 
            Assert.IsNotNull(template);

            StackPanel stackPanel = new StackPanel(); 
            listBoxItem.Template = template;
            ListBox testListBox =
#if WPFIMPLEMENTATION 
                new ListBox(); 
#else
                new HelperListBox(); 
#endif
            testListBox.Items.Add(listBoxItem);
            stackPanel.Children.Add(testListBox); 
            stackPanel.Measure(new Size { Width = 100, Height = 100 });
            return stackPanel;
        } 
 
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter", Justification="Input parameter is always a string")]
        public static T LoadXamlFromString<T>(string xaml) 
        {
#if WPF
            using (System.IO.MemoryStream stream = new System.IO.MemoryStream(System.Text.Encoding.UTF8.GetBytes(xaml))) 
            {
                return (T)System.Windows.Markup.XamlReader.Load(stream);
            } 
#else 
            return (T)System.Windows.Markup.XamlReader.Load(xaml);
#endif 
        }
    }
} 
