﻿// Copyright © Microsoft Corporation. 
// This source is subject to the Microsoft Source License for Silverlight Controls (March 2008 Release).
// Please see http://go.microsoft.com/fwlink/?LinkID=111693 for details.
// All other rights reserved. 

using System;
using System.Windows; 
using System.Windows.Controls; 
using System.Windows.Input;
using Microsoft.VisualStudio.TestTools.UnitTesting; 

#if WPF
public class SilverlightControlTest 
{
    protected void CreateAsyncTest(FrameworkElement element, params Action[] actions)
    { 
        foreach (Action action in actions) 
        {
            action(); 
        }
    }
} 
public sealed class AsynchronousAttribute : Attribute { }
#else
using Microsoft.Silverlight.Testing; 
#endif 

#if WPF && !WPFIMPLEMENTATION 
namespace WPF
#else
namespace System.Windows.Controls.Test 
#endif
{
    /// <summary> 
    /// Unit tests for System.Windows.Controls.ListBox 
    /// </summary>
    [TestClass] 
    public sealed partial class ScrollViewerTest : SilverlightControlTest
    {
        [TestMethod] 
        public void Constructor()
        {
            ScrollViewer scrollViewer = new ScrollViewer(); 
            Assert.IsNotNull(scrollViewer); 
        }
 
        [TestMethod]
        [Asynchronous]
        public void InitialValues() 
        {
            ScrollViewer scrollViewer = new ScrollViewer();
            Assert.AreEqual(0, scrollViewer.HorizontalOffset); 
            Assert.AreEqual(0, scrollViewer.VerticalOffset); 
            Assert.AreEqual(0, scrollViewer.ViewportWidth);
            Assert.AreEqual(0, scrollViewer.ViewportHeight); 
            Assert.AreEqual(0, scrollViewer.ScrollableWidth);
            Assert.AreEqual(0, scrollViewer.ScrollableHeight);
            Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedHorizontalScrollBarVisibility); 
            Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedVerticalScrollBarVisibility);
            CreateAsyncTest(scrollViewer, delegate
            { 
                // These tests are here so that generic.xaml settings will have been applied 
                Assert.AreEqual(ScrollBarVisibility.Disabled, scrollViewer.HorizontalScrollBarVisibility);
                Assert.AreEqual(ScrollBarVisibility.Disabled, ScrollViewer.GetHorizontalScrollBarVisibility(scrollViewer)); 
                Assert.AreEqual(ScrollBarVisibility.Visible, scrollViewer.VerticalScrollBarVisibility);
                Assert.AreEqual(ScrollBarVisibility.Visible, ScrollViewer.GetVerticalScrollBarVisibility(scrollViewer));
            }); 
        }

        [TestMethod] 
        public void SetHorizontalScrollBarVisibility() 
        {
            ScrollViewer scrollViewer = new ScrollViewer(); 
            ScrollViewer.SetHorizontalScrollBarVisibility(scrollViewer, ScrollBarVisibility.Auto);
            Assert.AreEqual(ScrollBarVisibility.Auto, scrollViewer.HorizontalScrollBarVisibility);
        } 

        [TestMethod]
        public void SetVerticalScrollBarVisibility() 
        { 
            ScrollViewer scrollViewer = new ScrollViewer();
            ScrollViewer.SetVerticalScrollBarVisibility(scrollViewer, ScrollBarVisibility.Auto); 
            Assert.AreEqual(ScrollBarVisibility.Auto, scrollViewer.VerticalScrollBarVisibility);
        }
 
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void GetHorizontalScrollBarVisibilityNull() 
        { 
            ScrollViewer.GetHorizontalScrollBarVisibility(null);
        } 

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))] 
        public void GetVerticalScrollBarVisibilityNull()
        {
            ScrollViewer.GetVerticalScrollBarVisibility(null); 
        } 

        [TestMethod] 
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetHorizontalScrollBarVisibilityNull()
        { 
            ScrollViewer.SetHorizontalScrollBarVisibility(null, ScrollBarVisibility.Hidden);
        }
 
        [TestMethod] 
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetVerticalScrollBarVisibilityNull() 
        {
            ScrollViewer.SetVerticalScrollBarVisibility(null, ScrollBarVisibility.Hidden);
        } 

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))] 
        public void SetComputedHorizontalScrollBarVisibilityWidth() 
        {
            ScrollViewer scrollViewer = new ScrollViewer(); 
            scrollViewer.SetValue(ScrollViewer.ComputedHorizontalScrollBarVisibilityProperty, Visibility.Collapsed);
        }
 
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SetScrollableWidth() 
        { 
            ScrollViewer scrollViewer = new ScrollViewer();
            scrollViewer.SetValue(ScrollViewer.ScrollableWidthProperty, 1.0); 
        }

        [TestMethod] 
        [ExpectedException(typeof(InvalidOperationException))]
        public void SetHorizontalOffset()
        { 
            ScrollViewer scrollViewer = new ScrollViewer(); 
            scrollViewer.SetValue(ScrollViewer.HorizontalOffsetProperty, 1.0);
        } 

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))] 
        public void SetViewportWidth()
        {
            ScrollViewer scrollViewer = new ScrollViewer(); 
            scrollViewer.SetValue(ScrollViewer.ViewportWidthProperty, 1.0); 
        }
 
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SetComputedHorizontalScrollBarVisibilityHeight() 
        {
            ScrollViewer scrollViewer = new ScrollViewer();
            scrollViewer.SetValue(ScrollViewer.ComputedHorizontalScrollBarVisibilityProperty, Visibility.Collapsed); 
        } 

        [TestMethod] 
        [ExpectedException(typeof(InvalidOperationException))]
        public void SetScrollableHeight()
        { 
            ScrollViewer scrollViewer = new ScrollViewer();
            scrollViewer.SetValue(ScrollViewer.ScrollableHeightProperty, 1.0);
        } 
 
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))] 
        public void SetVerticalOffset()
        {
            ScrollViewer scrollViewer = new ScrollViewer(); 
            scrollViewer.SetValue(ScrollViewer.VerticalOffsetProperty, 1.0);
        }
 
        [TestMethod] 
        [ExpectedException(typeof(InvalidOperationException))]
        public void SetViewportHeight() 
        {
            ScrollViewer scrollViewer = new ScrollViewer();
            scrollViewer.SetValue(ScrollViewer.ViewportHeightProperty, 1.0); 
        }

        [TestMethod] 
        [Asynchronous] 
        public void ContentFitsAuto()
        { 
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Auto, ScrollBarVisibility.Auto, 50, 50);
            CreateAsyncTest(scrollViewer, delegate
            { 
                scrollViewer.UpdateLayout();
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedHorizontalScrollBarVisibility);
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedVerticalScrollBarVisibility); 
            }); 
        }
 
        [TestMethod]
        [Asynchronous]
        public void ContentTooWideAuto() 
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Auto, ScrollBarVisibility.Auto, 500, 50);
            CreateAsyncTest(scrollViewer, delegate 
            { 
                scrollViewer.UpdateLayout();
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedHorizontalScrollBarVisibility); 
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedVerticalScrollBarVisibility);
            });
        } 

        [TestMethod]
        [Asynchronous] 
        public void ContentTooTallAuto() 
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Auto, ScrollBarVisibility.Auto, 50, 500); 
            CreateAsyncTest(scrollViewer, delegate
            {
                scrollViewer.UpdateLayout(); 
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedHorizontalScrollBarVisibility);
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedVerticalScrollBarVisibility);
            }); 
        } 

        [TestMethod] 
        [Asynchronous]
        public void ContentTooBigAuto()
        { 
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Auto, ScrollBarVisibility.Auto, 500, 500);
            CreateAsyncTest(scrollViewer, delegate
            { 
                scrollViewer.UpdateLayout(); 
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedHorizontalScrollBarVisibility);
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedVerticalScrollBarVisibility); 
            });
        }
 
        [TestMethod]
        [Asynchronous]
        public void ContentFitsVisible() 
        { 
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Visible, ScrollBarVisibility.Visible, 50, 50);
            CreateAsyncTest(scrollViewer, delegate 
            {
                scrollViewer.UpdateLayout();
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedHorizontalScrollBarVisibility); 
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedVerticalScrollBarVisibility);
            });
        } 
 
        [TestMethod]
        [Asynchronous] 
        public void ContentTooBigVisible()
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Visible, ScrollBarVisibility.Visible, 500, 500); 
            CreateAsyncTest(scrollViewer, delegate
            {
                scrollViewer.UpdateLayout(); 
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedHorizontalScrollBarVisibility); 
                Assert.AreEqual(Visibility.Visible, scrollViewer.ComputedVerticalScrollBarVisibility);
            }); 
        }

        [TestMethod] 
        [Asynchronous]
        public void ContentFitsHidden()
        { 
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Hidden, ScrollBarVisibility.Hidden, 50, 50); 
            CreateAsyncTest(scrollViewer, delegate
            { 
                scrollViewer.UpdateLayout();
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedHorizontalScrollBarVisibility);
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedVerticalScrollBarVisibility); 
            });
        }
 
        [TestMethod] 
        [Asynchronous]
        public void ContentTooBigHidden() 
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Hidden, ScrollBarVisibility.Hidden, 500, 500);
            CreateAsyncTest(scrollViewer, delegate 
            {
                scrollViewer.UpdateLayout();
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedHorizontalScrollBarVisibility); 
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedVerticalScrollBarVisibility); 
            });
        } 

        [TestMethod]
        [Asynchronous] 
        public void ContentFitsDisabled()
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Disabled, ScrollBarVisibility.Disabled, 50, 50); 
            CreateAsyncTest(scrollViewer, delegate 
            {
                scrollViewer.UpdateLayout(); 
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedHorizontalScrollBarVisibility);
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedVerticalScrollBarVisibility);
            }); 
        }

        [TestMethod] 
        [Asynchronous] 
        public void ContentTooBigDisabled()
        { 
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Disabled, ScrollBarVisibility.Disabled, 500, 500);
            CreateAsyncTest(scrollViewer, delegate
            { 
                scrollViewer.UpdateLayout();
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedHorizontalScrollBarVisibility);
                Assert.AreEqual(Visibility.Collapsed, scrollViewer.ComputedVerticalScrollBarVisibility); 
            }); 
        }
 
        [TestMethod]
        [Asynchronous]
        public void ScrollToHorizontalOffset() 
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Visible, ScrollBarVisibility.Visible, 500, 500);
            CreateAsyncTest(scrollViewer, delegate 
            { 
                Assert.AreEqual(0, scrollViewer.HorizontalOffset);
                scrollViewer.ScrollToHorizontalOffset(scrollViewer.ScrollableWidth); 
                Assert.AreEqual(scrollViewer.ScrollableWidth, scrollViewer.HorizontalOffset);
            });
        } 

        [TestMethod]
        [Asynchronous] 
        public void ScrollToVerticalOffset() 
        {
            ScrollViewer scrollViewer = LayoutScrollViewerWithContent(100, 100, ScrollBarVisibility.Visible, ScrollBarVisibility.Visible, 500, 500); 
            CreateAsyncTest(scrollViewer, delegate
            {
                Assert.AreEqual(0, scrollViewer.VerticalOffset); 
                scrollViewer.ScrollToVerticalOffset(scrollViewer.ScrollableHeight);
                Assert.AreEqual(scrollViewer.ScrollableHeight, scrollViewer.VerticalOffset);
            }); 
        } 

#if !WPFIMPLEMENTATION  // ScrollInDirection 

        [TestMethod]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1702:CompoundWordsShouldBeCasedCorrectly", MessageId = "InDirection", Justification = "Testing public method of same name")] 
        public void ScrollInDirection()
        {
            ScrollViewer scrollViewer = new ScrollViewer(); 
            scrollViewer.ScrollInDirection(Key.Up); 
            scrollViewer.ScrollInDirection(Key.Down);
            scrollViewer.ScrollInDirection(Key.Left); 
            scrollViewer.ScrollInDirection(Key.Right);
        }
 
#endif

        [TestMethod] 
        public void ScrollContentPresenterWithoutParent() 
        {
            ScrollContentPresenter scrollContentPresenter = new ScrollContentPresenter(); 
            scrollContentPresenter.Measure(new Size(100, 100));
            scrollContentPresenter.Arrange(new Rect(0, 0, 100, 100));
        } 

        private static ScrollViewer LayoutScrollViewerWithContent(double viewerWidth, double viewerHeight,
            ScrollBarVisibility viewerHorizontalVisibility, ScrollBarVisibility viewerVerticalVisibility, 
            double contentWidth, double contentHeight) 
        {
            return LayoutScrollViewerWithContent( 
                new ScrollViewer
                {
                    Width = viewerWidth, 
                    Height = viewerHeight,
                    HorizontalScrollBarVisibility = viewerHorizontalVisibility,
                    VerticalScrollBarVisibility = viewerVerticalVisibility 
                }, 
                new Canvas
                { 
                    Width = contentWidth,
                    Height = contentHeight
                }); 
        }

        private static ScrollViewer LayoutScrollViewerWithContent(ScrollViewer scrollViewer, object content) 
        { 
            scrollViewer.Content = content;
#if WPF 
            using (System.IO.MemoryStream stream = new System.IO.MemoryStream(System.Text.Encoding.UTF8.GetBytes(ScrollViewerTemplate)))
            {
                scrollViewer.Template = (ControlTemplate)System.Windows.Markup.XamlReader.Load(stream); 
            }
            Rect bounds = new Rect(0, 0, 1000, 1000);
            scrollViewer.Measure(new Size(bounds.Width, bounds.Height)); 
            scrollViewer.Arrange(bounds); 
#endif
            return scrollViewer; 
        }

#if WPF 
        private const string ScrollViewerTemplate =
            "<ControlTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\" xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\" " +
#if !WPFIMPLEMENTATION  // LB-SV-WPF: Changed assembly name
                "xmlns:controls=\"clr-namespace:WPF;assembly=LB-SV-WPF\" " + 
#endif
                "TargetType=\"{x:Type " + 
#if !WPFIMPLEMENTATION
                "controls:" +
#endif 
                "ScrollViewer}\">" +
                "<Grid>" +
                    "<" + 
#if !WPFIMPLEMENTATION 
                        "controls:" +
#endif 
                        "ScrollContentPresenter x:Name=\"ScrollContentPresenterElement\" Grid.Column=\"0\" Grid.Row=\"0\" Content=\"{TemplateBinding Content}\" ContentTemplate=\"{TemplateBinding ContentTemplate}\"/>" +
                    "<ScrollBar x:Name=\"VerticalScrollBarElement\" Grid.Column=\"1\" Grid.Row=\"0\"/>" +
                    "<ScrollBar x:Name=\"HorizontalScrollBarElement\" Grid.Column=\"0\" Grid.Row=\"1\" Orientation=\"Horizontal\"/>" + 
                "</Grid>" +
            "</ControlTemplate>";
#endif 
    } 
}
